@@ -5,12 +5,12 @@ import subprocess, signal
 import sys
 
 import warnings
-import cjson
 from copy import copy, deepcopy
 from flask import session, g
 from tiramisu.option import *
 from tiramisu.value import Multi
 from tiramisu.error import PropertiesOptionError, ValueWarning
+from tiramisu.setting import owners
 
 from creole.loader import creole_loader, config_load_store, \
                           config_save_values, config_get_values, \
@@ -20,6 +20,10 @@ from creole.var_loader import modes, modes_level, convert_value, type_option, \
 from creole.config import configeol
 from creole import eosfunc
 
+
+if sys.version_info[0] >= 3:
+    xrange = range
+
 try:
     from collections import OrderedDict
     from os import umask
@@ -29,9 +33,18 @@ except:
     from pyeole.odict import OrderedDict
 
 try:
-    from zephir.lib_zephir import xmlrpclib
+    from zephir.lib_zephir.xmlrpclib import ProtocolError
 except:
-    import xmlrpclib
+    try:
+        from xmlrpclib import ProtocolError
+    except:
+        # python3
+        from xmlrpc.client import ProtocolError
+try:
+    # python3
+    from importlib import reload
+except:
+    pass
 
 #from pyeole.log import init_logging
 #
@@ -69,13 +82,10 @@ def quit_genconfig(id_):
     :param id_: session id
     :type  id_: `str`
     """
-    p = subprocess.Popen(['ps', '-U', 'genconfig', '--no-header'], stdout=subprocess.PIPE)
+    p = subprocess.Popen(['ps', '-U', 'genconfig', '--no-header', '-o', 'pid,cmd'], stdout=subprocess.PIPE)
     out, err = p.communicate()
     for line in out.splitlines():
-        if 'uzbl-core' in line:
-            pid = int(line.split(None, 1)[0])
-            kill(pid, signal.SIGKILL)
-        elif 'chromium' in line:
+        if b'uzbl-core' in line or b'chromium' in line or b'chrome' in line or b'firefox' in line:
             pid = int(line.split(None, 1)[0])
             kill(pid, signal.SIGKILL)
 
@@ -101,7 +111,7 @@ def init_session(id_):
     :param id_: session id
     :type  id_: `str`
     """
-    if not session_tiramisu.has_key(id_):
+    if not id_ in session_tiramisu:
         session_tiramisu[id_] = {'config':None, 'mode': 'normal', 'debug': False, 'zephir': { 'available': False, 'reauth': True }}
 
 def init_zephir_session(id_, zephir):
@@ -213,10 +223,10 @@ def get_zephir_config(id_):
         code, res = proxy.serveurs.get_status(zephir['server_id'])
         code, info_serv = proxy.serveurs.get_serveur(zephir['server_id'])
 
-    except xmlrpclib.ProtocolError:
-        print "Accès au serveur %s non autorisé" % zephir['server_id']
-    except Exception, e:
-        print "Erreur de lecture des informations sur Zéphir \n(%s)" % str(e)
+    except ProtocolError:
+        print("Accès au serveur %s non autorisé" % zephir['server_id'])
+    except Exception as e:
+        print("Erreur de lecture des informations sur Zéphir \n(%s)" % str(e))
     else:
         if res['config_ok'] == 0:
             raise ConfigEmptyError("La configuration distante (Zéphir) n'existe pas")
@@ -302,8 +312,6 @@ def get_load_errors(id_):
             try:
                 varname = config.unwrap_from_path('.'.join(path), force_permissive=True).impl_getdoc()
                 varid = path[-1]
-                if isinstance(category_name, unicode):
-                    category_name = unicode.encode(category_name, 'utf8')
                 load_errors.append(MSG_ERR_IMPORT_VAR.format(category_name, varname, varid))
             except PropertiesOptionError:
                 pass
@@ -340,6 +348,8 @@ def get_categories(id_, get_nested=False, init=False, zephir_sync=False):
     else:
         config = get_config(id_)
     ret = []
+    if get_nested:
+        config.cfgimpl_get_settings().remove('cache')
     for name, ownconfig in config.creole.iter_groups():
         optdesc = ownconfig.cfgimpl_get_description()
         help_ = optdesc.impl_get_information('help', '')
@@ -350,7 +360,7 @@ def get_categories(id_, get_nested=False, init=False, zephir_sync=False):
                     'icon': icon,
                     'help': help_}
         has_value = False
-        for var in ownconfig.iter_all():
+        for var in list(ownconfig.iter_all()):
             path = get_path(name, var[0])
             option = config.unwrap_from_path(path)
             type_ = type_option[type(option)]
@@ -375,6 +385,8 @@ def get_categories(id_, get_nested=False, init=False, zephir_sync=False):
                     category['mode'] = mode
                     break
             ret.append(category)
+    if get_nested:
+        config.cfgimpl_get_settings().append('cache')
     return ret
 
 
@@ -414,7 +426,7 @@ def get_path(category_name, name=None, master_slave=None):
 
 
 def get_variable(id_, category_name, name, slave=None, slaves=None, current_tag=False,
-                 force_permissive=False):
+                 force_permissive=False, from_diff=False):
     """ Get variable from specified name
     :param id_: session id
     :type  id_: `str`
@@ -448,14 +460,23 @@ def get_variable(id_, category_name, name, slave=None, slaves=None, current_tag=
             with warnings.catch_warnings(record=True) as warns:
                 value = config._getattr(path, force_permissive=force_permissive)
             if isinstance(value, Multi):
-                value = list(value)
+                if from_diff:
+                    val_list = []
+                    for val_ in value:
+                        if isinstance(val_, PropertiesOptionError):
+                            val_list.append(None)
+                        else:
+                            val_list.append(val_)
+                    value = val_list
+                else:
+                    value = list(value)
             if is_slave:
                 is_default_owner = None
                 is_calculated =  option.impl_has_callback()
             else:
                 is_default_owner = config.cfgimpl_get_values().is_default_owner(option, force_permissive=True)
                 is_calculated =  option.impl_has_callback() and is_default_owner
-        except ValueError, err:
+        except ValueError as err:
             is_default_owner = True
             is_calculated = False
             error = err
@@ -586,12 +607,13 @@ def get_variable(id_, category_name, name, slave=None, slaves=None, current_tag=
                 slave = copy(slave)
                 slave['id'] = slave['id'] + '_' + str(idx)
                 slave['value'] = slave['value'][idx]
-                slave.pop('multi')
-                slave['groupid'] = vname
-                slave['index'] = idx
-                slave['default_owner'] = config.cfgimpl_get_values().is_default_owner(opt_slave, index=idx, force_permissive=True)
-                slave['owner'] = str(config.getowner(opt_slave, index=idx, force_permissive=True))
-                var['slaves'].append(slave)
+                if not isinstance(slave['value'], PropertiesOptionError):
+                    slave.pop('multi')
+                    slave['groupid'] = vname
+                    slave['index'] = idx
+                    slave['default_owner'] = config.cfgimpl_get_values().is_default_owner(opt_slave, index=idx, force_permissive=True)
+                    slave['owner'] = str(config.getowner(opt_slave, index=idx, force_permissive=True))
+                    var['slaves'].append(slave)
             _vars.append(var)
         variable = {'name': vname, 'id': vname,
                     'description': variable['description'], 'masters': _vars,
@@ -917,7 +939,7 @@ def get_mode(id_):
     return session_tiramisu[id_]['mode']
 
 
-def get_variable_from_path(id_, path, force_permissive=False):
+def get_variable_from_path(id_, path, force_permissive=False, from_diff=False):
     """ Get variable from a path
     :param id_: session id
     :type  id_: `str`
@@ -937,7 +959,7 @@ def get_variable_from_path(id_, path, force_permissive=False):
         name = path[2]
         slave = path[3]
     return get_variable(id_, category_name, name, slave=slave,
-                        force_permissive=force_permissive)
+                        force_permissive=force_permissive, from_diff=from_diff)
 
 
 def valid_mandatory(id_, zephir_sync=False):
@@ -1020,11 +1042,15 @@ def diff_config(id_, zephir_sync=False):
                             new_value = []
                             new_owner = []
                             if master in orig_values:
-                                len_master_value = len(orig_values[master]['val'])
+                                master_value = orig_values[master]['val']
                             else:
-                                len_master_value = len(dico['val'])
+                                master_value = dico['val']
+                            if not isinstance(master_value, list):
+                                len_master_value = 0
+                            else:
+                                len_master_value = len(master_value)
                             for idx in xrange(len_master_value):
-                                if str(idx) in dico['val']:
+                                if isinstance(dico['val'], dict) and str(idx) in dico['val']:
                                     new_value.append(dico['val'][str(idx)])
                                     new_owner.append(dico['owner'][str(idx)])
                                 else:
@@ -1041,6 +1067,19 @@ def diff_config(id_, zephir_sync=False):
 
     setting = config.cfgimpl_get_settings()
     new_values = config.cfgimpl_get_values().get_modified_values()
+    force_store_values = config.impl_get_information('force_store_values', [])
+    value_changed = False
+    for path in force_store_values:
+        if path not in new_values:
+            value_changed = True
+            try:
+                config.setattr(path, config.getattr(path, force_permissive=True), force_permissive=True)
+                opt = config.unwrap_from_path(path, force_permissive=True)
+                config.cfgimpl_get_values().setowner(opt, owners.forced)
+            except PropertiesOptionError:
+                pass
+    if value_changed:
+        new_values = config.cfgimpl_get_values().get_modified_values()
     for path, own_val in new_values.items():
         owner, value = own_val
         if not isinstance(config.getattr(path, returns_raise=True, force_permissive=True), PropertiesOptionError):
@@ -1089,11 +1128,12 @@ def diff_config(id_, zephir_sync=False):
                 path = config.creole.find_first(byname=key, type_='path',
                                                 force_permissive=True)
                 diffs.append(dict(
-                    get_variable_from_path(id_, path,
-                                           force_permissive=True).items() + {
+                    list(get_variable_from_path(id_, path,
+                                                force_permissive=True,
+                                                from_diff=True).items()) + list({
                         'old_value': store[key]['val'],
                         'old_user': store[key]['owner']
-                    }.items()
+                    }.items())
                 ))
             except AttributeError:
                 #remove older/disabled variable's
@@ -1103,11 +1143,12 @@ def diff_config(id_, zephir_sync=False):
                 path = config.creole.find_first(byname=key, type_='path',
                                                 force_permissive=True)
                 diffs.append(dict(
-                    get_variable_from_path(id_, path,
-                                           force_permissive=True).items() + {
+                    list(get_variable_from_path(id_, path,
+                                                force_permissive=True,
+                                                from_diff=True).items()) + list({
                         'new_value': new_store[key]['val'],
                         'new_user': new_store[key]['owner']
-                    }.items()
+                    }.items())
                 ))
             except AttributeError:
                 #for example when imported config file
@@ -1123,13 +1164,14 @@ def diff_config(id_, zephir_sync=False):
                     path = config.creole.find_first(byname=key, type_='path',
                                                     force_permissive=True)
                     diffs.append(dict(
-                        get_variable_from_path(id_, path,
-                                               force_permissive=True).items() + {
+                        list(get_variable_from_path(id_, path,
+                                                    force_permissive=True,
+                                                    from_diff=True).items()) + list({
                             'old_value': store[key]['val'],
                             'old_user': store[key]['owner'],
                             'new_value': new_store[key]['val'],
                             'new_user': new_store[key]['owner']
-                        }.items()
+                        }.items())
                     ))
                 except AttributeError:
                     pass
@@ -1185,8 +1227,8 @@ def save_zephir_config(id_):
     zephir = get_zephir(id_)
     values = get_values(id_)
     try:
-        code, msg = zephir['proxy'].serveurs.save_conf(zephir['server_id'], [unicode(str(values))])
-    except Exception, e:
+        code, msg = zephir['proxy'].serveurs.save_conf(zephir['server_id'], [str(values)])
+    except Exception as e:
         # FIXME: Log Error messages
         code = 0
         msg = str(e)
